unit MacapiDemo.AppDelegates;

{                                                                           }
{ File:       MacapiDemo.AppDelegates                                       }
{ Function:   AppDelegate implementation for SampleNiblessApp.              }
{             Demonstrates the use of the Macapi.* units in Delphi XE2.     }
{ Language:   Delphi XE2 and above                                          }
{ Author:     Rudy Velthuis                                                 }
{             original author: Tim Lindquist (Tim.Lindquist@asu.edu),       }
{                              ASU Polytechnic, Engineering                 }
{ Copyright:  (c) 2011 Rudy Velthuis                                        }
{ Original code copyright:                                                  }
{             (c) 2010 Tim Lindquist                                        }
{ Version:    1.0                                                           }
{ References: http://pooh.poly.asu.edu/Cst420                               }
{                                                                           }
{ Disclaimer: Redistribution and use in source and binary forms, with or    }
{             without modification, are permitted provided that the         }
{             following conditions are met:                                 }
{                                                                           }
{             * Redistributions of source code must retain the above        }
{               copyright notice, this list of conditions and the following }
{               disclaimer.                                                 }
{             * Redistributions in binary form must reproduce the above     }
{               copyright notice, this list of conditions and the following }
{               disclaimer in the documentation and/or other materials      }
{               provided with the distribution.                             }
{                                                                           }
{               THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER "AS IS"   }
{               AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT   }
{               LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND   }
{               FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO      }
{               EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE   }
{               FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,   }
{               OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,    }
{               PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,   }
{               DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED  }
{               AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT }
{               LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)      }
{               ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF }
{               ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                  }
{                                                                           }

interface

uses
  System.SysUtils, Macapi.AppKit, Macapi.ObjCRuntime, Macapi.ObjectiveC,
  Macapi.Foundation, Macapi.CocoaTypes;

type
  AppDelegate = interface(NSApplicationDelegate)
    ['{2E84EB44-1E75-45C8-A641-29F2D7D53189}']
    procedure accumulateFires(theTimer: NSTimer); cdecl;
    function applicationShouldTerminate(sender: Pointer): Integer; cdecl;
    procedure applicationWillFinishLaunching(notification: Pointer); cdecl;
    procedure comboBoxSelectionDidChange(notification: Pointer); cdecl;
    procedure debug(aMessage: NSString); cdecl;
    procedure endIt; cdecl;
    procedure saveLib; cdecl;
  end;

  TAppDelegate = class(TOCLocal, AppDelegate, NSComboBoxDelegate)
  protected
    window: NSWindow;
    view: NSView;
    titleLab,
    authorLab,
    commentLab,
    urlLab,
    albumLab,
    categoryLab,
    subcategoryLab: NSTextField;
    titleTB,
    authorTB,
    commentTB,
    urlTB,
    albumTB: NSTextField;
    titleCB,
    categoryCB,
    subcategoryCB: NSComboBox;
    aTimer: NSTimer;
    fires: Integer;
  public
    // Delegate methods (must be cdecl)
    procedure applicationDidFinishLaunching(Notification: Pointer); cdecl;
    function applicationShouldTerminate(sender: Pointer): Integer; cdecl;
    procedure applicationWillTerminate(Notification: Pointer); cdecl;
    procedure applicationWillFinishLaunching(notification: Pointer); cdecl;
    procedure comboBoxSelectionDidChange(notification: Pointer); cdecl;

    // Action methods (must be cdecl)
    procedure accumulateFires(theTimer: NSTimer); cdecl;
    procedure endIt; cdecl;
    procedure saveLib; cdecl;

    // Utility methods
    procedure addButtonWithTitle(const aTitle: String; target: Pointer;
      const action: string; rect: NSRect);
    function addFieldWithTitle(const aTitle: string; at: NSRect): NSTextField;
    function addLabelWithTitle(const aTitle: string; at: NSRect): NSTextField;
    procedure debug(aMessage: NSString); cdecl;

    // Constructor and destructor
    constructor Create;
    destructor Destroy; override;
  end;

implementation

// Unlike NSStr, this returns an unwrapped (raw Objective C) NSString.
function RawStr(const S: string): Pointer;
begin
  Result := TNSString.OCClass.stringWithUTF8String(PAnsiChar(UTF8String(S)));
end;

{ TAppDelegate }

// - (void) accumulateFires: (NSTimer *) theTimer
procedure TAppDelegate.accumulateFires(theTimer: NSTimer);
begin
  // fires = fires+1;
  Inc(fires);
  // [commentTB setStringValue:
  //     [NSString stringWithFormat:@"Timer fired %d times.",fires]];
  commentTB.setStringValue(NSStr(Format('Timer fired %d times.', [fires])));
end;

// - (void) addButtonWithTitle: (NSString *) aTitle target: (id) target action: (SEL) action rect: (NSRect) rect
procedure TAppDelegate.addButtonWithTitle(const aTitle: string;
  target: Pointer; const action: string; rect: NSRect);
var
  button: NSButton;
begin
  // NSButton *button = [[NSButton alloc] initWithFrame: rect];
  button := TNSButton.Wrap(TNSButton.alloc.initWithFrame(rect));
  // [button setTitle: aTitle];
  button.setTitle(NSStr(aTitle));
  // [button setAction: action];
  button.setAction(sel_getUid(PAnsiChar(UTF8String(action))));
  // [button setTarget: target];
  button.setTarget(target);
  // [button setButtonType: NSMomentaryPushButton];
  button.setButtonType(NSMomentaryPushButton);
  // [button setBezelStyle: NSRoundedBezelStyle];
  button.setBezelStyle(NSRoundedBezelStyle);
  // [[window contentView] addSubView: button];
  TNSView.Wrap(window.contentView).addSubView(button);
end;

// - (NSTextField *) addFieldWithTitle: (NSString *) aTitle at: (NSRect) rect
function TAppDelegate.addFieldWithTitle(const aTitle: string;
  at: NSRect): NSTextField;
begin
  // NSTextField *result = [[NSTextField alloc] initWithFrame: rect];
  Result := TNSTextField.Wrap(TNSTextField.Alloc.initWithFrame(at));
  // [result setSelectable: YES];
  Result.setSelectable(True);
  // [result setEditable: YES];
  Result.setEditable(True);
  // [result setBezeled: YES];
  Result.setBezeled(True);
  // [result setDrawsBackground: YES];
  Result.setDrawsBackground(True);
  // [result setStringValue: aTitle];
  Result.setStringValue(NSStr(aTitle));
  // [[window contentView] addSubView: result];
  TNSView.Wrap(window.contentView).addSubview(Result);
  // return result;
end;

// - (NSTextField *) addLabelWithTitle: (NSString *) aTitle at: (NSRect) rect
function TAppDelegate.addLabelWithTitle(const aTitle: string;
  at: NSRect): NSTextField;
begin
  // NSTextField *result = [[NSTextField alloc] initWithFrame: rect];
  Result := TNSTextField.Wrap(TNSTextField.Alloc.initWithFrame(at));
  // [result setSelectable: NO];
  Result.setSelectable(False);
  // [result setBezeled: NO];
  Result.setBezeled(False);
  // [result setDrawsBackground: NO];
  Result.setDrawsBackground(False);
  // [result setStringValue: aTitle];
  Result.setStringValue(NSStr(aTitle));
  // [[window contentView] addSubView: result];
  TNSView.Wrap(window.contentView).addSubview(Result);
  // return result;
end;

// - (void) applicationDidFinishLaunching: (NSNotification *) notification
procedure TAppDelegate.applicationDidFinishLaunching(Notification: Pointer);
begin
  // [window setContentView: view];
  window.setContentView(view);
  // [window makeKeyAndOrderFront: self];
  window.makeKeyAndOrderFront(Self);
  // aTimer = [NSTimer scheduledTimerWithInterval: 2.0f
  //                                       target: self
  //                                     selector: @selector('accumulateFires:')
  //                                     userInfo: nil
  //                                      repeats: YES];
  aTimer := TNSTimer.Wrap(
    TNSTimer.OCClass.scheduledTimerWithTimeInterval(
      2.0, Self.GetObjectID, sel_getUid('accumulateFires:'), nil, True));
end;

function TAppDelegate.applicationShouldTerminate(sender: Pointer): Integer;
begin
  // return YES;
  Result := YES;
end;

procedure TAppDelegate.applicationWillFinishLaunching(notification: Pointer);
begin
  Writeln('applicationWillFinishLaunching');
  window.setContentView(view);
end;

procedure TAppDelegate.applicationWillTerminate(Notification: Pointer);
begin
  // nothing; required.
end;

procedure TAppDelegate.comboBoxSelectionDidChange(notification: Pointer);
var
  selected: string;
begin
  Writeln('Combobox selection changed');
  // NSString*selected = [subcategoryCB objectValueOfSelectedItem];
  selected := string(TNSString.Wrap(subcategoryCB.objectValueOfSelectedItem).UTF8String);
  // [albumTB setStringValue:[NSString stringWithFormat:@"You selected: %s", [selected UTF8String]]];
  albumTB.setStringValue(NSStr(Format('You selected: %s', [selected])));
end;

constructor TAppDelegate.Create;
var
  contentSize: NSRect;
begin
  // self = [super init];
  inherited Create;
  // if (self)
  // {
  //     fires = 0;
  fires := 0;

  //     NSRect contentSize = NSMakeRect(100, 150, 560, 300);
  contentSize := MakeNSRect(100, 150, 560, 300);
  //     window = [[NSWindow alloc] initWithContentRect: contentSize
  //                                          styleMask: NSTitledWindowMask
  //                                            backing: NSBackingStoreBuffered
  //                                              defer: YES];
  window := TNSWindow.Wrap(TNSWindow.alloc.initWithContentRect(contentSize, NSTitledWindowMask, NSBackingStoreBuffered, True));
  //     [window setTitle: @"Dr. Lindquist's example GUI"];
  window.setTitle(NSStr('Dr. Lindquist''s example GUI'));
  //     view = [[NSView alloc] initWithFrame: contentSize];
  view := TNSView.Wrap(TNSView.Alloc.initWithFrame(contentSize));
  //     [window setContentView: view];
  window.setContentView(view);

  // FWIW: window.contentView does not return the same address as view. Why? I don't know.
  Writeln(Format('window.contentView: %p, view: %.8x', [window.contentView, NativeUInt(view)]));
  // Output: window.contentView: 00210900, view: 00243DD0

  //     // Add the buttons across the bottom of the view.
  //     // Place the button at 20,20 (x,y) in window with width 82 and height 32

  //     [self addButtonWithTitle: @"Exit" target: self action: selector(endIt) rect: NSMakeRect(20, 20, 82, 32)];
  addButtonWithTitle('Exit', Self.GetObjectID, 'endIt', MakeNSRect(20, 20, 82, 32));
  //     [self addButtonWithTitle: @"Save" target: self action: selector(saveLib) rect: NSMakeRect(108, 20, 82, 32)];
  addButtonWithTitle('Save', Self.GetObjectID, 'saveLib', MakeNSRect(108, 20, 82, 32));
  //     [self addButtonWithTitle: @"Restore" target: self action: selector(saveLib) rect: NSMakeRect(196, 20, 82, 32)];
  addButtonWithTitle('Restore', Self.GetObjectID, 'saveLib', MakeNSRect(196, 20, 82, 32));
  //     [self addButtonWithTitle: @"Add" target: self action: selector(saveLib) rect: NSMakeRect(284, 20, 82, 32)];
  addButtonWithTitle('Add', Self.GetObjectID, 'saveLib', MakeNSRect(284, 20, 82, 32));
  //     [self addButtonWithTitle: @"Remove" target: self action: selector(saveLib) rect: NSMakeRect(372, 20, 82, 32)];
  addButtonWithTitle('Remove', Self.GetObjectID, 'saveLib', MakeNSRect(372, 20, 82, 32));
  //     [self addButtonWithTitle: @"Edit" target: self action: selector(saveLib) rect: NSMakeRect(460, 20, 82, 32)];
  addButtonWithTitle('Edit', Self.GetObjectID, 'saveLib', MakeNSRect(460, 20, 82, 32));

  //     // Create the labels, text fields and combo boxes for manipulating media
  //     titleLab = [self addLabelWithTitle: @"Title" at: NSmakeRect(20, 250, 30, 20)];
  titleLab := addLabelWithTitle('Title', MakeNSRect(20, 250, 30, 20));

  //     titleCB = [[NSComboBox alloc] initWithFrame: NSMakeRect(60, 250, 200, 25)];
  titleCB := TNSComboBox.Wrap(TNSComboBox.alloc.initWithFrame(MakeNSRect(90, 250, 170, 25)));
  //     [titleCB removeAllItems];
  titleCB.removeAllItems;

  //     [titleCB addItemWithObjectValue: @"Title One"];
  titleCB.addItemWithObjectValue(RawStr('Title one'));
  //     [titleCB addItemWithObjectValue: @"Title Two"];
  titleCB.addItemWithObjectValue(RawStr('Title two'));
  //     [titleCB selectItemAtIndex: 0];
  titleCB.selectItemAtIndex(0);
  //     [[window contentView] addSubView: titleCB];
  TNSView.Wrap(window.contentView).addSubview(titleCB);

  //     authorLab = [self addLabelWithTitle: @"Author" at: NSMakeRect(280, 250, 50, 20)];
  authorLab := addLabelWithTitle('Author', MakeNSRect(280, 250, 50, 20));
  //     authorTB = [self addFieldWithTitle: @"anAuthor" at: NSMakeRect(340, 250, 200, 20)];
  authorTB := addFieldWithTitle('anAuthor', MakeNSRect(365, 250, 170, 23));

  //     commentLab = [self addLabelWithTitle: @"Comment" at: NSMakeRect(20, 200, 70, 20)];
  commentLab := addLabelWithTitle('Comment', MakeNSRect(20, 200, 70, 20));
  //     commentTB = [self addFieldWithTitle: @"aComment" at: NSMakeRect(90, 200, 450, 20)];
  commentTB := addFieldWithTitle('aComment', MakeNSRect(90, 200, 445, 23));

  //     urlLab = [self addLabelWithTitle: @"URL" at: NSMakeRect(20, 150, 30, 20)];
  urlLab := addLabelWithTitle('URL', MakeNSRect(20, 150, 30, 20));
  //     urlTB = [self addFieldWithTitle: @"aURL/File" at: NSMakeRect(60, 150, 200, 20)];
  urlTB := addFieldWithTitle('aURL/File', MakeNSRect(90, 150, 170, 23));

  //     albumLab = [self addLabelWithTitle: @"Album" at: NSMakeRect(280, 150, 50, 20)];
  albumLab := addLabelWithTitle('Album', MakeNSRect(280, 150, 50, 20));
  //     urlTB = [self addFieldWithTitle: @"anAlbum" at: NSMakeRect(330, 150, 200, 20)];
  albumTB := addFieldWithTitle('anAlbum', MakeNSRect(365, 150, 170, 23));

  //     categoryLab = [self addLabelWithTitle: @"Category" at: NSMakeRect(20, 90, 60, 20)];
  categoryLab := addLabelWithTitle('Category', MakeNSRect(20, 90, 60, 20));
  //     categoryCB = [[NSComboBox alloc] initWithFrame: NSMakeRect(90, 90, 175, 25)];
  categoryCB := TNSComboBox.Wrap(TNSComboBox.Alloc.initWithFrame(MakeNSRect(90, 90, 175, 25)));
  //     [[window contentView] addSubView: categoryCB];
  TNSView.Wrap(window.contentView).addSubview(categoryCB);
  //     [categoryCB removeAllItems];
  categoryCB.removeAllItems;
  //     [categoryCB addItemWithObjectValue: @"Category One"];
  categoryCB.addItemWithObjectValue(RawStr('Category one'));
  //     [categoryCB addItemWithObjectValue: @"Category Two"];
  categoryCB.addItemWithObjectValue(RawStr('Category two'));
  //     [categoryCB selectItemAtIndex: 0];
  categoryCB.selectItemAtIndex(0);

  //     subcategoryLab = [self addLabelWithTitle: @"Subcategory" at: NSMakeRect(280, 90, 85, 20)];
  subcategoryLab := addLabelWithTitle('Subcategory', MakeNSRect(280, 90, 85, 20));
  //     subcategoryCB = [[NSComboBox alloc] initWithFrame: NSMakeRect(365, 90, 175, 25)];
  subcategoryCB := TNSComboBox.Wrap(TNSComboBox.Alloc.initWithFrame(MakeNSRect(365, 90, 175, 25)));
  //     [subcategoryCB removeAllItems];
  subcategoryCB.removeAllItems;
  //     [subcategoryCB addItemWithObjectValue:@"Drama"];
  subcategoryCB.addItemWithObjectValue(RawStr('Drama'));
  //     [subcategoryCB addItemWithObjectValue:@"Comedy"];
  subcategoryCB.addItemWithObjectValue(RawStr('Comedy'));
  //     [subcategoryCB addItemWithObjectValue:@"Musical"];
  subcategoryCB.addItemWithObjectValue(RawStr('Musical'));
  //     [subcategoryCB selectItemWithObjectValue:@"Musical"];
  subcategoryCB.selectItemWithObjectValue(RawStr('Musical'));

  //     // to handle selection changes in a combobox, set the delegate
  //     // and implement the method
  //     // - (void) comboBoxSelectionDidChange: (NSNotification *) notification;

  //    [subcategoryCB setDelegate:self];
  subcategoryCB.setDelegate(self);
  //    [[window contentView] addSubview: subcategoryCB];
  TNSView.Wrap(window.contentView).addSubView(subcategoryCB);
  // }
  // return self;
end;

//procedure TAppDelegate.dealloc;
//begin
//
//end;

procedure TAppDelegate.debug(aMessage: NSString);
begin
  // not important - Writeln will write to console.
end;

destructor TAppDelegate.Destroy;
begin
  // [titleLab release];
  titleLab.release;
  // [view release];
  view.release;
  // [window release];
  window.release;
  // [super dealloc];
  inherited;
end;

procedure TAppDelegate.endIt;
begin
  // [NSApp terminate: self];
  TNSApplication.Wrap(TNSApplication.OCClass.sharedApplication).terminate(self);
end;

procedure TAppDelegate.saveLib;
begin
  // [authorTB setStringValue:(NSString*)([titleCB itemObjectValueAtIndex: [titleCB indexOfSelectedItem]])];
  authorTB.setStringValue(TNSString.Wrap(titleCB.itemObjectValueAtIndex(titleCB.indexOfSelectedItem)));
  // [self debug:[NSString stringWithFormat:@"SaveLib called: %s\n", [@"copying to authorTB." UTF8String]]];
  // not used.
end;

end.
