#import "AppDelegate.h"
#import <AppKit/NSButton.h>

// to write debug messages to a file, change NO below to YES.
// Appends messages sent to the debug: method at the end of the file
// .../SampleNiblessApp/NiblessApp.app/DebugMessageLog.txt (on Mac).
// on GNUstep, you must manually copy DebugMessageLog.txt into the
// NiblessAppGNUstep.app directory before executing with debugon yes.
#define DEBUGON NO

/**
 * Purpose: demonstrate GUI construction on Cocoa without using the
 * interface builder. This program is executable on both MacOSX and on
 * Windows running GNUstep.
 * Cst420 Foundations of Distributed Applications
 * see http://pooh.poly.asu.edu/Cst420
 * @author Tim Lindquist (Tim.Lindquist@asu.edu), ASU Polytechnic, Engineering
 * with credits to Casper B Hansen for structure of nibless of apps.
 * @version November 2010
 */
@implementation AppDelegate

- (id) init {
   if ( (self = [super init]) ) {
      fires = 0;
      // create a reference rect
      // set this window at position 100,150 (x,y) from lower left
      // with size 560,300 (x,y)
      NSRect contentSize = NSMakeRect(100.0f, 150.0f, 560.0f, 300.0f);
      // allocate window
      window = [[NSWindow alloc] initWithContentRect:contentSize
                                           styleMask:NSTitledWindowMask
                                             backing:NSBackingStoreBuffered
                                               defer:YES];
      [window setTitle:@"Dr. Lindquist's example GUI"];
      // allocate view
      view = [[NSView alloc] initWithFrame:contentSize];
      [window setContentView:view];

      //Add the buttons across the bottom of the view.
      // place the button at 20,20 (x,y) in window with width 75 and height 30
      [self addButtonWithTitle:@"Exit" target:self action:@selector(endIt)
                          rect:NSMakeRect(20,20,75,30)];
      [self addButtonWithTitle:@"Save" target:self action:@selector(saveLib)
                          rect:NSMakeRect(110,20,75,30)];
      [self addButtonWithTitle:@"Restore" target:self action:@selector(saveLib)
                          rect:NSMakeRect(200,20,75,30)];
      [self addButtonWithTitle:@"Add" target:self action:@selector(saveLib)
                          rect:NSMakeRect(290,20,75,30)];
      [self addButtonWithTitle:@"Remove" target:self action:@selector(saveLib)
                          rect:NSMakeRect(380,20,75,30)];
      [self addButtonWithTitle:@"Edit" target:self action:@selector(saveLib)
                          rect:NSMakeRect(470,20,75,30)];

      // create the labels, text fields and combo boxes for manipulating media
      titleLab = [self addLabelWithTitle:@"Title" at:NSMakeRect(20,250,30,20)];
      [titleLab retain];
      titleCB = [[NSComboBox alloc] initWithFrame: NSMakeRect(60,250,200,25)];
      [titleCB retain];
      [titleCB removeAllItems];
      [titleCB addItemWithObjectValue:@"Title One"];
      [titleCB addItemWithObjectValue:@"Title Two"];
      [titleCB selectItemAtIndex:0];
      [[window contentView] addSubview: titleCB];
      authorLab = [self addLabelWithTitle:@"Author"
                                       at:NSMakeRect(280,250,50,20)];
      [authorLab retain];
      authorTB = [self addFieldWithTitle:@"anAuthor"
                                       at:NSMakeRect(340,250,200,20)];
      [authorTB retain];
      commentLab = [self addLabelWithTitle:@"Comment"
                                       at:NSMakeRect(20,200,70,20)];
      [commentLab retain];
      commentTB = [self addFieldWithTitle:@"aComment"
                                       at:NSMakeRect(90,200,450,20)];
      [commentTB retain];
      urlLab = [self addLabelWithTitle:@"URL" at:NSMakeRect(20,150,30,20)];
      [urlLab retain];
      urlTB = [self addFieldWithTitle:@"aURL/File"
                                   at:NSMakeRect(60,150,200,20)];
      [urlTB retain];
      albumLab = [self addLabelWithTitle:@"Album"
                                       at:NSMakeRect(280,150,50,20)];
      [albumLab retain];
      albumTB = [self addFieldWithTitle:@"anAlbum"
                                       at:NSMakeRect(330,150,200,20)];
      [albumTB retain];

      categoryLab = [self addLabelWithTitle:@"Category"
                                         at:NSMakeRect(20,90,60,20)];
      [titleLab retain];
      categoryCB = [[NSComboBox alloc]
                                      initWithFrame: NSMakeRect(90,90,175,25)];
      [categoryCB retain];
      [categoryCB removeAllItems];
      [categoryCB addItemWithObjectValue:@"Category One"];
      [categoryCB addItemWithObjectValue:@"Category Two"];
      [categoryCB selectItemAtIndex:0];
      [[window contentView] addSubview: categoryCB];

      subcategoryLab = [self addLabelWithTitle:@"Subcategory"
                                      at:NSMakeRect(280,90,85,20)];
      [subcategoryLab retain];
      subcategoryCB = [[NSComboBox alloc]
                                     initWithFrame: NSMakeRect(365,90,175,25)];
      [subcategoryCB retain];
      [subcategoryCB removeAllItems];
      [subcategoryCB addItemWithObjectValue:@"Drama"];
      [subcategoryCB addItemWithObjectValue:@"Comedy"];
      [subcategoryCB addItemWithObjectValue:@"Musical"];
      [subcategoryCB selectItemWithObjectValue:@"Musical"];
      // to handle selection changes in a combobox, set the delegate
      // and implement the method
      //- (void) comboBoxSelectionDidChange: (NSNotification*)notification;
      [subcategoryCB setDelegate:self];
      [[window contentView] addSubview: subcategoryCB];
   }
   return self;
}

- (void) applicationWillFinishLaunching:(NSNotification *)notification {
   // attach the view to the window
   [window setContentView:view];
}

- (void) applicationDidFinishLaunching:(NSNotification *)notification {
   // note that printf's or NSLogs do not display on the console. Write to GUI.
   // [urlTB setStringValue:@"In application did finish launching."];
   // make the window visible.
   [window makeKeyAndOrderFront:self];
   // schedule a timer with the run loop that will call 
   NSRunLoop * myRunLoop = [NSRunLoop currentRunLoop];
   aTimer = [[NSTimer scheduledTimerWithTimeInterval:2.0
                                    target:self
                                  selector:@selector(accumulateFires:)
                                  userInfo:nil
                                   repeats:YES] retain];
}

- (void) accumulateFires:(NSTimer*) theTimer{
   fires = fires+1;
   [commentTB setStringValue:[NSString
                            stringWithFormat:@"Timer fired %d times.",fires]];
}

- (void) saveLib {
   [authorTB setStringValue:(NSString*)([titleCB itemObjectValueAtIndex:
                                            [titleCB indexOfSelectedItem]])];
   [self debug:[NSString stringWithFormat:@"SaveLib called: %s\n",
                         [@"copying to authorTB." UTF8String]]];
}

- (void) addButtonWithTitle: (NSString*)aTitle
                     target: (id) anObject
                     action: (SEL) anAction
                       rect: (NSRect) aRect {
      NSButton *button = [[[NSButton alloc] initWithFrame:aRect ] autorelease];
      [button setTitle:aTitle];
      [button setAction:anAction];
      [button setTarget: anObject];
      [button setButtonType:NSMomentaryPushButton];
      [button setBezelStyle:NSTexturedSquareBezelStyle];
      [[window contentView] addSubview: button];
}

- (NSTextField*) addLabelWithTitle:(NSString*) aTitle at: (NSRect) aRect {
   NSTextField* label =[[[NSTextField alloc] initWithFrame: aRect] autorelease];
   [label setSelectable: NO];
   [label setBezeled: NO];
   [label setDrawsBackground: NO];
   [label setStringValue: aTitle];
   [[window contentView] addSubview: label];
   return label;
}

- (NSTextField*) addFieldWithTitle:(NSString*) aTitle at: (NSRect) aRect {
   NSTextField* label =[[[NSTextField alloc] initWithFrame: aRect] autorelease];
   [label setSelectable: YES];
   [label setEditable: YES];
   [label setBezeled: YES];
   [label setDrawsBackground: YES];
   [label setStringValue: aTitle];
   [[window contentView] addSubview: label];
   return label;
}

- (void) endIt {
   [self debug:[NSString stringWithFormat:@"Exit button pushed: %s\n",
                         [@"Completing program." UTF8String]]];
   [NSApp terminate:self];
   }

- (BOOL) applicationShouldTerminate:(id) sender {
   return YES;
}

- (void) comboBoxSelectionDidChange: (NSNotification*)notification {
   NSString*selected = [subcategoryCB objectValueOfSelectedItem];
   [albumTB setStringValue:[NSString stringWithFormat:@"You selected: %s",
                                     [selected UTF8String]]];
}

- (void) debug: (NSString*) aMessage{
   if(DEBUGON){
      //NSBundle*aBundle = [NSBundle bundleForClass:[self class]];
      NSBundle*aBundle = [NSBundle mainBundle];
      // path to application: .../SampleNiblessApp/NiblessApp.app
      NSString*pathSeg = [aBundle bundlePath];
      NSString * fileName = [NSString stringWithFormat:@"%s/%s",
                     [pathSeg UTF8String],[@"DebugMessageLog.txt" UTF8String]];
      NSFileHandle* fh = [NSFileHandle fileHandleForWritingAtPath: fileName];
      [fh seekToEndOfFile];
      [fh writeData: [aMessage dataUsingEncoding:NSUTF8StringEncoding]];
      [fh closeFile];
       //printf("debug: %s\n", [aMessage UTF8String]);
   }
}

- (void)dealloc {
   // don’t forget to release allocated objects!
   [titleLab release];
   [view release];
   [window release];
   [super dealloc];
}

@end
