program SampleNiblessApp;

{                                                                           }
{ File:       SampleNiblessApp                                              }
{ Function:   Sample Delphi app that creates a window without using .nib or }
{             .xib files. Demonstrates how to create instances of classes   }
{             and how to call class functions. Uses AppKit to generate      }
{             Cocoa-style Objective-C "native" controls.                    }
{ Language:   Delphi XE2 and above                                          }
{ Author:     Rudy Velthuis                                                 }
{             original author: Tim Lindquist (Tim.Lindquist@asu.edu),       }
{                              ASU Polytechnic, Engineering                 }
{ Copyright:  (c) 2011 Rudy Velthuis                                        }
{ Version:    1.0                                                           }
{ References: http://pooh.poly.asu.edu/Cst420                               }
{                                                                           }
{ Disclaimer: Redistribution and use in source and binary forms, with or    }
{             without modification, are permitted provided that the         }
{             following conditions are met:                                 }
{                                                                           }
{             * Redistributions of source code must retain the above        }
{               copyright notice, this list of conditions and the following }
{               disclaimer.                                                 }
{             * Redistributions in binary form must reproduce the above     }
{               copyright notice, this list of conditions and the following }
{               disclaimer in the documentation and/or other materials      }
{               provided with the distribution.                             }
{                                                                           }
{               THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER "AS IS"   }
{               AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT   }
{               LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND   }
{               FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO      }
{               EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE   }
{               FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,   }
{               OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,    }
{               PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,   }
{               DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED  }
{               AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT }
{               LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)      }
{               ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF }
{               ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                  }
{                                                                           }

{ Original comment of Objective C program:

  Purpose: demonstrate GUI construction on Cocoa without using the
  interface builder. This program is executable on MacOSX.
  Cst420 Foundations of Distributed Applications
  see http://pooh.poly.asu.edu/Cst420
  @author Tim Lindquist (Tim.Lindquist@asu.edu), ASU Polytechnic, Engineering
  with credits to Casper B Hansen for structure of nibless of apps.
  @version November 2010
}

{$APPTYPE CONSOLE}

{$R *.res}

uses
  System.SysUtils,
  Macapi.AppKit,
  Macapi.CoreFoundation,
  Macapi.Foundation,
  Macapi.ObjCRuntime,
  Macapi.CocoaTypes,
  MacapiDemo.AppDelegates in 'MacapiDemo.AppDelegates.pas';

procedure Main;
var
  pool: NSAutoReleasePool;
  application: NSApplication;
  applicationDelegate: AppDelegate;
begin
  // NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
  pool := TNSAutoReleasePool.Create;
  try
    // NSApplication *application = [NSApplication sharedApplication];
    application := TNSApplication.Wrap(TNSApplication.OCClass.sharedApplication);
    // AppDelegate *applicationDelegate = [[AppDelegate alloc] init];
    applicationDelegate := TAppDelegate.Create;
    // application.delegate = applicationDelegate;
    application.setDelegate(applicationDelegate);
    // [applicaton run];
    application.run;
  finally
    // [pool drain];
    pool.drain;
  end;
end;

var
  AppKitModule: HModule;

begin
  try
    AppKitModule := LoadLibrary('/System/Library/Frameworks/AppKit.framework/AppKit');
    try
      Main;
    finally
      FreeLibrary(AppKitModule);
    end;
  except
    on E: Exception do
      Writeln(E.ClassName, ': ', E.Message);
  end;
end.
